package binarysearchtree;

import binarytree.*;

public class BinarySearchTree extends BinaryTreeBasis {
    
    public BinarySearchTree() {
    }

    public BinarySearchTree(KeyedItem rootItem) {
	super(rootItem);
    }

    public void insert(KeyedItem newItem) {
	root = insertItem(root, newItem);
    }

    public KeyedItem retrieve(Comparable searchKey) {
	return retrieveItem(root, searchKey);
    }

    public void delete(Comparable searchKey) throws TreeException {
	root = deleteItem(root, searchKey);
    }

    public void delete(KeyedItem item) throws TreeException {
	root = deleteItem(root, item.getKey());
    }

    protected TreeNode insertItem(TreeNode tNode,
				  KeyedItem newItem) {
	TreeNode newSubtree;
	
	if(tNode == null) {
	    tNode = new TreeNode(newItem, null, null);
	    return tNode;
	}
	KeyedItem nodeItem = (KeyedItem)tNode.getItem();

	if(newItem.getKey().compareTo(nodeItem.getKey())<0) {
	    newSubtree = insertItem(tNode.getLeft(), newItem);
	    tNode.setLeft(newSubtree);
	    return tNode;
	} else {
	    newSubtree = insertItem(tNode.getRight(), newItem);
	    tNode.setRight(newSubtree);
	    return tNode;
	}
    }


    protected KeyedItem retrieveItem(TreeNode tNode,
				     Comparable searchKey) {
	KeyedItem treeItem;
	if(tNode == null) {
	    treeItem = null;
	} else {
	    KeyedItem nodeItem = (KeyedItem)tNode.getItem();
	    if(searchKey.compareTo(nodeItem.getKey()) == 0) {
		treeItem = (KeyedItem)tNode.getItem();
	    } else if(searchKey.compareTo(nodeItem.getKey()) < 0) {
		treeItem = retrieveItem(tNode.getLeft(), searchKey);
	    } else {
		treeItem = retrieveItem(tNode.getRight(), searchKey);
	    }
	}
	return treeItem;
    }

    protected TreeNode deleteItem(TreeNode tNode,
				  Comparable searchKey) {
	TreeNode newSubtree;
	if(tNode == null) {
	    throw new TreeException("TreeException: item not found");
	} else {
	    KeyedItem nodeItem = (KeyedItem)tNode.getItem();
	    if(searchKey.compareTo(nodeItem.getKey()) == 0) {
		tNode = deleteNode(tNode);
	    } else if(searchKey.compareTo(nodeItem.getKey()) < 0) {
		newSubtree = deleteItem(tNode.getLeft(), searchKey);
		tNode.setLeft(newSubtree);
	    } else {
		newSubtree = deleteItem(tNode.getRight(), searchKey);
		tNode.setRight(newSubtree);
	    }
	}

	return tNode;
    }

    protected TreeNode deleteNode(TreeNode tNode) {
	KeyedItem replacementItem;

	if( (tNode.getLeft() == null)
	    && (tNode.getRight() == null) ) {
	    return null;
	} else if (tNode.getLeft() == null) {
	    return tNode.getRight();
	} else if (tNode.getRight() == null) {
	    return tNode.getLeft();
	} else {
	    replacementItem = findLeftmost(tNode.getRight());
	    tNode.setItem(replacementItem);
	    tNode.setRight(deleteLeftmost(tNode.getRight()));
	    return tNode;
	}
    }

    protected KeyedItem findLeftmost(TreeNode tNode) {
	if(tNode.getLeft() == null) {
	    return (KeyedItem)tNode.getItem();
	} else {
	    return findLeftmost(tNode.getLeft());
	}
    }

    protected TreeNode deleteLeftmost(TreeNode tNode) {
	if(tNode.getLeft() == null) {
	    return tNode.getRight();
	} else {
	    tNode.setLeft(deleteLeftmost(tNode.getLeft()));
	    return tNode;
	}
    }
}
