package binarytree;

public class BinaryTree extends BinaryTreeBasis {

    public BinaryTree() {
    }

    public BinaryTree(Object rootItem) {
	super(rootItem);
    }

    public BinaryTree(Object rootItem, 
		      BinaryTree leftTree,
		      BinaryTree rightTree) {
	root = new TreeNode(rootItem, null, null);
	attachLeftSubtree(leftTree);
	attachRightSubtree(rightTree);
    }

    public void setRootItem(Object newItem) {
	if(root != null) {
	    root.setItem(newItem);
	} else {
	    root = new TreeNode(newItem, null, null);
	}
    }

    public void attachLeft(Object newItem) 
	throws TreeException{
	if(isEmpty()) {
	    throw new TreeException(
	    "TreeException: attachLeft() on empty tree!");
	} else if(root.getLeft() != null) {
	    throw new TreeException(
	    "TreeException: attachLeft() over existing left subtree!");
	} else {
	    root.setLeft(new TreeNode(newItem, null, null));
	}
    }

    public void attachRight(Object newItem) 
	throws TreeException {
	if(isEmpty()) {
	    throw new TreeException(
	    "TreeException: attachRight() on empty tree!");

	} else if(root.getRight() != null) {
	    throw new TreeException(
	    "TreeException: attachRight() over existing right subtree!");
	} else {
	    root.setRight(new TreeNode(newItem, null, null));
	}
    }

    public void attachLeftSubtree(BinaryTree leftTree)
	throws TreeException {
	if(isEmpty()) {
	    throw new TreeException(
	    "TreeException: attachLeftSubtree on empty tree");
	} else if(root.getLeft() != null) {
	    throw new TreeException(
	    "TreeException: attachLeftSubtree over existing left subtree!");
	} else {
	    root.setLeft(leftTree.root);
	    leftTree.makeEmpty();
	}
    }

    public void attachRightSubtree(BinaryTree rightTree) 
    throws TreeException {
	if(isEmpty()) {
	    throw new TreeException(
	    "TreeException: attachRightSubtree on empty tree");
	} else if(root.getRight() != null) {
	    throw new TreeException(
	    "TreeException: attachRightSubtree over existing right subtree!");
	} else {
	    root.setRight(rightTree.root);
	    rightTree.makeEmpty();
	}
    }

    protected BinaryTree(TreeNode rootNode) {
	root = rootNode;
    }

    public BinaryTree detachLeftSubtree()
	throws TreeException {
	if(isEmpty()) {
	    throw new TreeException(
	    "TreeException: detachLeftSubtree() on empty tree");
	} else {
	    BinaryTree leftTree;
	    leftTree = new BinaryTree(root.getLeft());
	    root.setLeft(null);
	    return leftTree;
	}
    }

    public BinaryTree detachRightSubtree()
	throws TreeException {
	if(isEmpty()) {
	    throw new TreeException(
	    "TreeException: detachRightSubtree() on empty tree");
	} else {
	    BinaryTree rightTree;
	    rightTree = new BinaryTree(root.getRight());
	    root.setRight(null);
	    return rightTree;
	}
    }
}
